@@ -1,4 +1,4 @@
-; $Id: nat.scm,v 1.12 2007/07/25 10:33:17 schwicht Exp $
+; $Id: nat.scm,v 1.15 2008/01/25 13:30:21 logik Exp $
 (display "loading nat.scm ...") (newline)
 
 ; (load "~/minlog/init.scm")
@@ -54,15 +54,15 @@
 
 ; Program constants.
 
-(add-program-constant "NatPlus" (py "nat=>nat=>nat") t-deg-one)
-(add-program-constant "NatTimes" (py "nat=>nat=>nat") t-deg-one)
-(add-program-constant "NatLt" (py "nat=>nat=>boole") t-deg-one)
-(add-program-constant "NatLe" (py "nat=>nat=>boole") t-deg-one)
-(add-program-constant "Pred" (py "nat=>nat") t-deg-one)
-(add-program-constant "NatMinus" (py "nat=>nat=>nat") t-deg-one)
-(add-program-constant "NatMax" (py "nat=>nat=>nat") t-deg-one)
-(add-program-constant "NatMin" (py "nat=>nat=>nat") t-deg-one)
-(add-program-constant "AllBNat" (py "nat=>(nat=>boole)=>boole") t-deg-one)
+(add-program-constant "NatPlus" (py "nat=>nat=>nat") t-deg-zero)
+(add-program-constant "NatTimes" (py "nat=>nat=>nat") t-deg-zero)
+(add-program-constant "NatLt" (py "nat=>nat=>boole") t-deg-zero)
+(add-program-constant "NatLe" (py "nat=>nat=>boole") t-deg-zero)
+(add-program-constant "Pred" (py "nat=>nat") t-deg-zero)
+(add-program-constant "NatMinus" (py "nat=>nat=>nat") t-deg-zero)
+(add-program-constant "NatMax" (py "nat=>nat=>nat") t-deg-zero)
+(add-program-constant "NatMin" (py "nat=>nat=>nat") t-deg-zero)
+(add-program-constant "AllBNat" (py "nat=>(nat=>boole)=>boole") t-deg-zero)
 
 ; Tokens used by the parser.
 
@@ -196,6 +196,34 @@
 
 ; Properties of NatPlus
 
+; "NatPlusTotal"
+(set-goal "Total NatPlus")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(ind)
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+(assume "nat2" "IH")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(ng)
+(use (make-proof-in-aconst-form (finalg-to-total-to-e-aconst (py "nat"))))
+(use "IH")
+; Proof finished
+(save "NatPlusTotal")
+
+
+; (set-goal "all nat1,nat2 E(nat1+nat2)")
+; (assume "nat1")
+; (ind)
+;   (use "Truth-Axiom")
+; (assume "nat2" "IH")
+; (use "IH")
+; (add-theorem "NatPlusTotal")
+
+
 (set-goal "all nat 0+nat=nat")
 (ind)
   (use "Truth-Axiom")
@@ -238,6 +266,41 @@
 
 ; Properties of NatTimes
 
+; "NatTimesTotal"
+(set-goal "Total NatTimes")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(ind)
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (use "Truth-Axiom")
+(assume "nat2" "IH")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(ng)
+(use (make-proof-in-aconst-form (finalg-to-total-to-e-aconst (py "nat"))))
+(use "Total" 'left)
+(use "Total" 'left)
+(use "NatPlusTotal")
+(use "IH")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+; Proof finished
+(save "NatTimesTotal")
+
+; (set-goal "all nat1,nat2 E(nat1*nat2)")
+; (assume "nat1")
+; (ind)
+;   (use "Truth-Axiom")
+; (assume "nat2" "IH")
+; (use "Allpartial-All-nat")
+
+; (use "NatPlusTotal")
+; (use "IH")
+; (add-theorem "NatTimesTotal")
+
+
 (set-goal "all nat 0*nat=0")
 (ind)
   (use "Truth-Axiom")
@@ -403,6 +466,30 @@
 
 ; Properties of NatLt
 
+; "NatLtTotal"
+(set-goal "Total NatLt")
+(assert (pf "all nat2,nat1 Total(nat1<nat2)"))
+  (ind)
+  (assume "nat1")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+  (use "Truth-Axiom")
+  (assume "nat2" "IH")
+  (cases)
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+  (use "Truth-Axiom")
+  (use "IH")
+(assume "H")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "nat2")
+(use "H")
+; Proof finished.
+(save "NatLtTotal")
+
+
 (set-goal "all nat nat<Succ nat")
 (ind)
   (use "Truth-Axiom")
@@ -479,6 +566,30 @@
 
 ; Properties of NatLe
 
+; "NatLeTotal"
+(set-goal "Total NatLe")
+(assert (pf "all nat1,nat2 Total(nat1<=nat2)"))
+  (ind)
+  (assume "nat2")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+  (use "Truth-Axiom")
+  (assume "nat1" "IH")
+  (cases)
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+  (use "Truth-Axiom")
+  (use "IH")
+(assume "H")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "nat2")
+(use "H")
+; Proof finished.
+(save "NatLeTotal")
+
+
 (set-goal "all nat nat<=nat")
 (ind)
   (use "Truth-Axiom")
@@ -626,6 +737,44 @@
 (save "NatLeToLtSucc")
 
 
+; "NatLtToSuccLe"
+(set-goal (pf "all nat1,nat2(nat1<nat2 -> Succ nat1<=nat2)"))
+(ind)
+  (cases)
+  (assume "Absurd")
+  (use "Efq")
+  (use "Absurd")
+  (assume "nat2" "Trivial")
+  (use "Truth-Axiom")
+(assume "nat1" "IH1")
+  (cases)
+  (assume "Absurd")
+  (use "Efq")
+  (use "Absurd")
+(use "IH1")
+; Proof finished.
+(save "NatLtToSuccLe")
+
+
+; "NatSuccLeToLt"
+(set-goal (pf "all nat1,nat2(Succ nat1<=nat2 -> nat1<nat2)"))
+(ind)
+  (cases)
+  (assume "Absurd")
+  (use "Efq")
+  (use "Absurd")
+  (assume "nat2" "Trivial")
+  (use "Truth-Axiom")
+(assume "nat1" "IH1")
+  (cases)
+  (assume "Absurd")
+  (use "Efq")
+  (use "Absurd")
+(use "IH1")
+; Proof finished.
+(save "NatSuccLeToLt")
+
+
 ; "NatLeCases"
 (set-goal "all nat2,nat1(nat1<=nat2 -> (nat1<nat2 -> Pvar) -> 
                         (nat1=nat2 -> Pvar) -> Pvar)")
@@ -673,6 +822,41 @@
 
 ; Properties of NatMinus and Pred
 
+; "PredTotal"
+(set-goal (pf "Total Pred"))
+(use "Total")
+(use "All-AllPartial")
+(cases)
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+(assume "nat")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+; Proof finished.
+(save "PredTotal")
+
+
+; "NatMinusTotal"
+(set-goal "Total NatMinus")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(ind)
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+(assume "nat2" "IH")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(ng)
+(use (make-proof-in-aconst-form (finalg-to-total-to-e-aconst (py "nat"))))
+(use "Total" 'left)
+(use "PredTotal")
+(use "IH")
+; Proof finished
+(save "NatMinusTotal")
+
+
 (set-goal (pf "all nat1,nat2 Pred(Succ nat1--nat2)=nat1--nat2"))
 (assume "nat1")
 (ind)
@@ -705,6 +889,34 @@
 
 ; Properties of NatMax
 
+; "NatMaxTotal"
+(set-goal "Total NatMax")
+(assert (pf "all nat2,nat1 Total(nat1 max nat2)"))
+  (ind)
+  (assume "nat1")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (use "Truth-Axiom")
+  (assume "nat2" "IH")
+  (cases)
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (use "Truth-Axiom")
+  (assume "nat1")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (ng)  
+  (use (make-proof-in-aconst-form (finalg-to-total-to-e-aconst (py "nat"))))
+  (use "IH")
+(assume "H")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "nat2")
+(use "H")
+; Proof finished.
+(save "NatMaxTotal")
+
+
 (set-goal "all nat 0 max nat=nat")
 (cases)
   (use "Truth-Axiom")
@@ -782,6 +994,34 @@
 
 ; Properties of NatMin
 
+; "NatMinTotal"
+(set-goal "Total NatMin")
+(assert (pf "all nat2,nat1 Total(nat1 min nat2)"))
+  (ind)
+  (assume "nat1")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (use "Truth-Axiom")
+  (assume "nat2" "IH")
+  (cases)
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (use "Truth-Axiom")
+  (assume "nat1")
+  (use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+  (ng)  
+  (use (make-proof-in-aconst-form (finalg-to-total-to-e-aconst (py "nat"))))
+  (use "IH")
+(assume "H")
+(use "Total")
+(use "All-AllPartial")
+(assume "nat1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "nat2")
+(use "H")
+; Proof finished.
+(save "NatMinTotal")
+
+
 (set-goal "all nat 0 min nat=0")
 (cases)
   (use "Truth-Axiom")
@@ -858,6 +1098,57 @@
 
 ; Properties of AllBNat
 
+; "AndConstTotal"
+(set-goal "Total AndConst")
+(use "Total")
+(use "All-AllPartial")
+(assume "boole1")
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "boole2")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+(use "Truth-Axiom")
+; Proof finished
+(save "AndConstTotal")
+
+
+; "AllBNatTotal"
+(set-goal (pf "Total AllBNat"))
+(use "Total")
+(use "All-AllPartial")
+(ind)
+; Base
+(use "Total" 'right)
+(use "All-AllPartial")
+(assume "nat=>boole")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "boole"))))
+(use "Truth-Axiom")
+; Step
+(assume "nat" "IH")
+(use "Total" 'right)
+(assume "(nat=>boole)^" "Tf")
+(ng)
+(assert (pf "all boole^1(Total boole^1 -> all boole^2(Total boole^2 -> 
+                         Total(boole^1 andb boole^2)))"))
+  (assume "boole^1" "T1" "boole^2" "T2")
+  (use "Total")
+  (use "Total")
+  (use "AndConstTotal")
+  (use "T1")
+  (use "T2")
+(assume "TA")
+(use "TA")
+(use "Total")
+(use "IH")
+(use "Tf")
+(use "Total")
+(use "Tf")
+(use (make-proof-in-aconst-form (finalg-to-e-to-total-aconst (py "nat"))))
+(use "Truth-Axiom")
+; Proof finished
+(save "AllBNatTotal")
+
+
 ; "AllBNatIntro"
 (set-goal "all (nat=>boole)^,nat2(
       all nat1(nat1<nat2 -> (nat=>boole)^nat1) -> 
@@ -903,4 +1194,8 @@
 ; Proof finished.
 (save "AllBNatElim")
 
+; For use with grec we add
+
+(define natlt-obj (nbe-term-to-object (pt "NatLt") '()))
+
 (av "n" "m" "k" (py "nat"))
