@@ -1,4 +1,4 @@
-; $Id: atr.scm,v 1.18 2007/08/24 08:34:23 schwicht Exp $
+; $Id: atr.scm,v 1.22 2008/01/28 09:20:10 logik Exp $
 
 ; 17. A-translation
 ; =================
@@ -394,12 +394,10 @@
 		 (make-proof-in-avar-form u1)
 		 (make-proof-in-avar-form u2)))))))
     (case (tag formula)
-      ((atom)
+      ((atom predicate)
        (let ((u (formula-to-new-avar formula)))
 	 (make-proof-in-imp-intro-form
 	  u (make-proof-in-avar-form u))))
-      ((predicate) ;falsity-log
-       (make-proof-in-avar-form F-to-bot-avar))
       ((imp)
        (let* ((prem (imp-form-to-premise formula)) ;irrelevant
 	      (concl (imp-form-to-conclusion formula)) ;also irrelevant
@@ -449,7 +447,7 @@
     (atr-relevant? formula)
     (myerror "atr-irrel-goal-proof" "irrelevant formula expected" formula)
     (case (tag formula)
-      ((atom)
+      ((atom predicate)
        (let ((u (formula-to-new-avar formula)))
 	 (make-proof-in-imp-intro-form
 	  u (make-proof-in-avar-form u))))
@@ -596,8 +594,8 @@
 		(cons u (append goal-avars (list proof-of-bot))))))))
 
 ; Given a proof of As -> Ds -> all ys(Gs -> bot) -> bot with As
-; arbitrary and Ds definite fromulas, we transform it into a proof of
-; (F -> bot) -> As -> Ds^F -> all ys(Gs^F -> bot) -> bot.
+; arbitrary, Ds definite and Gs goal formulas, we transform it into a
+; proof of (F -> bot) -> As -> Ds^F -> all ys(Gs^F -> bot) -> bot.
 
 (define (atr-min-excl-proof-to-bot-reduced-proof min-excl-proof)
   (let* ((formula (unfold-formula (proof-to-formula min-excl-proof)))
@@ -655,9 +653,10 @@
 	   (append (cons F-to-bot-avar us) (list u proof-of-bot)))))
 
 ; Substituting the formula ex ys Gs^F for bot in the proof given above
-; of (F -> bot) -> As' -> Ds^F -> all ys(Gs^F -> bot) -> bot, both the
+; of (F -> bot) -> As -> Ds^F -> all ys(Gs^F -> bot) -> bot, both the
 ; efq-premise and the wrong formula become provable and we obtain a
-; proof of As' -> Ds^F -> ex ys Gs^F.
+; proof of As' -> Ds^F -> ex ys Gs^F, where As' is defined to be
+; As[bot := ex ys Gs^F].
 
 (define (atr-min-excl-proof-to-intuit-ex-proof min-excl-proof)
   (let* ((formula (unfold-formula (proof-to-formula min-excl-proof)))
@@ -709,13 +708,11 @@
 				  (map make-proof-in-avar-form us)
 				  (list proof-of-wrong-formula-F))))))))
 
-; Let minl-excl-proof be a proof of
-; all params D_1 ->...D_n -> all y((G -> bot) -> bot) where
-; D_1,...,D_n are either definite or arbitrary formulas, and G is a
-; goal formula.
-; Let realizers t_i for the nondefinite formulas among D_1 ... D_n 
-; be given such that t_i mr D_i^atr-X.  The following procedure
-; construct an eterm such that Ds->G[y:=eterm].
+; Given a proof of As -> Ds -> all ys(Gs -> bot) -> bot with As
+; arbitrary, Ds definite and Gs goal formulas.  Let ss be realizers of
+; As' := As[bot := ex ys Gs^F] and ts be realizers of Ds^F, both under
+; the assumptions As and Ds.  The following procedure constructs an
+; eterm such that As -> Ds->Gs^F[ys:=eterm].
 
 (define (atr-min-excl-proof-to-structured-extracted-term
 	 min-excl-proof . realizers-for-nondefinite-formulas)
@@ -764,9 +761,12 @@
 	(myerror "min-excl-proof?" "falsity-log expected"
 		 (imp-form-to-final-conclusion final-kernel)))
     (if (not (apply and-op (map atr-goal? goals)))
-	(apply myerror
-	       (append (list "min-excl-proof?" "goal formulas expected")
-		       goals)))
+	(let ((not-atr-goals
+	       (list-transform-positive goals
+		 (lambda (goal) (not (atr-goal? goal))))))
+	  (apply myerror
+		 (append (list "min-excl-proof?" "goal formulas expected")
+			 not-atr-goals))))
     (if (not (apply and-op (map atr-definite? ds)))
 	(if COMMENT-FLAG
 	    (begin
