@@ -1,4 +1,4 @@
-; $Id: ets.scm,v 1.60 2007/09/10 08:22:57 schwicht Exp $
+; $Id: ets.scm,v 1.63 2008/01/28 09:20:10 logik Exp $
 ; 16. Extracted terms
 ; ===================
 
@@ -279,6 +279,9 @@
 			 (aconst-to-repro-formulas aconst))))
 		((string=? "Cases" name)
 		 (cases-aconst-to-if-term aconst))
+                ((string=? "GInd" name)
+                 (make-term-in-const-form
+                  (gind-aconst-to-et-grec-const aconst)))
 		((string=? "Intro" name)
 		 (make-term-in-const-form
 		  (apply number-and-idpredconst-to-et-constr
@@ -597,6 +600,23 @@
                          (make-term-in-var-form test-var)
                          (map make-term-in-var-form alt-vars))))))))
 
+(define (gind-aconst-to-et-grec-const aconst)
+  (let* ((uninst-gind-formula (aconst-to-uninst-formula aconst))
+         (measure-var-and-vars (all-form-to-vars uninst-gind-formula))
+         (vars (cdr measure-var-and-vars))
+         (m (length vars))
+         (tpinst (aconst-to-tpinst aconst))
+         (tsubst (list-transform-positive tpinst
+                   (lambda (x) (tvar-form? (car x)))))
+         (pvar-and-cterm
+          (list-search-positive tpinst
+            (lambda (x) (pvar-form? (car x)))))
+         (cterm (cadr pvar-and-cterm))
+         (et-type (formula-to-et-type (cterm-to-formula cterm)))
+         (arg-types (map var-to-type vars))
+         (type-info (append arg-types (list et-type))))
+    (type-info-etc-to-grec-const type-info tsubst)))
+
 (define (number-and-idpredconst-to-et-constr i idpc)
   (let* ((name (idpredconst-to-name idpc))
 	 (clauses (idpredconst-name-to-clauses name))
@@ -1334,6 +1354,189 @@
                  (cons (make-term-in-var-form var)
                        (map make-proof-in-avar-form avars))))))))))))))))
 
+(define (gind-aconst-to-mr-gind-proof aconst)
+  (let ((pvar-to-mr-pvar (make-pvar-to-mr-pvar)))
+    (gind-aconst-to-mr-gind-proof-aux pvar-to-mr-pvar aconst)))
+
+(define (gind-aconst-to-mr-gind-proof-aux pvar-to-mr-pvar aconst)
+  (let* ((uninst-gind-formula (aconst-to-uninst-formula aconst))
+         (all-formula (car (aconst-to-repro-formulas aconst)))
+         (free (formula-to-free all-formula))
+         (measure-var-and-vars (all-form-to-vars uninst-gind-formula))
+         (measure-var (car measure-var-and-vars))
+         (vars (cdr measure-var-and-vars))
+         (m (length vars))
+         (kernel-formula (all-form-to-final-kernel all-formula m)) ;A(xs)
+         (kernel-formula-et-type (formula-to-et-type kernel-formula)))
+    (if
+     (nulltype? kernel-formula-et-type)
+     (let* ((mr-all-formula
+             (real-and-formula-to-mr-formula-aux
+              'eps all-formula pvar-to-mr-pvar))
+            (gind-aconst (all-formula-to-gind-aconst mr-all-formula m)))
+       (make-proof-in-aconst-form gind-aconst))
+     (let* ((inst-formula (aconst-to-inst-formula aconst))
+            (prog-formula ;Prog_mu{xs | A(xs)}
+             (imp-form-to-premise (all-form-to-final-kernel inst-formula)))
+            (prog-formula-et-type (formula-to-et-type prog-formula))
+            (real-var (type-to-new-var prog-formula-et-type)) ;G
+            (real-term (make-term-in-var-form real-var))
+            (prog-avar ;v: G mr Prog_mu{xs | A(xs)}
+             (formula-to-new-avar
+              (real-and-formula-to-mr-formula-aux
+               real-term prog-formula pvar-to-mr-pvar)
+              "v"))
+            (types (map term-to-type vars))
+            (tpinst (aconst-to-tpinst aconst))
+            (tsubst (list-transform-positive tpinst
+                      (lambda (x) (tvar-form? (car x)))))
+            (type-info (append types (list kernel-formula-et-type)))
+            (grec-const
+             (type-info-etc-to-grec-const type-info tsubst all-formula))
+            (grec-term (make-term-in-const-form grec-const)) ;[[GInd]]
+            (grecguard-const
+             (type-info-etc-to-grecguard-const type-info tsubst all-formula))
+            (grecguard-term (make-term-in-const-form grecguard-const))
+            (applied-grec-term
+             (apply mk-term-in-app-form
+                    (cons grec-term
+                          (append (map make-term-in-var-form
+                                       measure-var-and-vars)
+                                  (list real-term)))))
+            (mr-kernel-formula ;GRec mu xs G mr A(xs)
+             (real-and-formula-to-mr-formula-aux
+              applied-grec-term kernel-formula pvar-to-mr-pvar))
+            (mr-all-formula ;all xs.GRec mu xs G mr A(xs)
+             (apply mk-all (append vars (list mr-kernel-formula))))
+            (mr-free (formula-to-free mr-all-formula))
+            (mr-gind-aconst
+             (all-formula-to-gind-aconst mr-all-formula m))
+            (mr-gind-inst-formula
+             (aconst-to-inst-formula mr-gind-aconst))
+            (mr-measure-var (all-form-to-var mr-gind-inst-formula))
+            (mr-prog-formula
+             (imp-form-to-premise
+              (all-form-to-final-kernel mr-gind-inst-formula)))
+            (subst-mr-prog-formula
+             (formula-subst mr-prog-formula mr-measure-var
+                            (make-term-in-var-form measure-var)))
+            (mr-prog-prem-avar ;u:all ys.mu ys < mu xs -> GRec mu ys G mr A(ys)
+             (formula-to-new-avar
+              (imp-form-to-premise
+               (all-form-to-final-kernel subst-mr-prog-formula)) "u"))
+            (new-vars (map var-to-new-var vars)) ;ys
+            (test-term ;mu ys < mu xs
+             (apply mk-term-in-app-form
+                    (list (if (not (assoc "nat" ALGEBRAS))
+			      (myerror "First execute (libload \"nat.scm\")")
+			      (pt "NatLt"))
+                          (apply mk-term-in-app-form
+                                 (map make-term-in-var-form
+                                      (cons measure-var new-vars)))
+                          (apply mk-term-in-app-form
+                                 (map make-term-in-var-form
+				      measure-var-and-vars)))))
+            (prev-applied-grecguard-term ;GRecGuard mu ys G (mu xs < mu ys)
+             (apply mk-term-in-app-form
+                    (cons grecguard-term
+                          (cons (make-term-in-var-form measure-var)
+                                (append (map make-term-in-var-form new-vars)
+                                        (list real-term test-term))))))
+            (test-var (type-to-new-var (py "boole")))
+            (prev-applied-grecguard-var-term
+             (apply mk-term-in-app-form
+                    (cons grecguard-term
+                          (cons (make-term-in-var-form measure-var)
+                                (append (map make-term-in-var-form new-vars)
+                                        (list
+                                         real-term
+                                         (make-term-in-var-form test-var)))))))
+            (vars-to-new-vars-subst
+             (make-substitution vars (map make-term-in-var-form new-vars)))
+            (subst-kernel-formula
+             (formula-substitute kernel-formula vars-to-new-vars-subst))
+            (cterm (make-cterm test-var
+                               (real-and-formula-to-mr-formula-aux
+                                prev-applied-grecguard-var-term
+                                subst-kernel-formula pvar-to-mr-pvar)))
+					;this is a hack ... is there no better
+					;way to do this substitution?
+            (tsubst-eq-compat-rev-proof
+             (proof-subst (make-proof-in-aconst-form eq-compat-rev-aconst)
+                          (car (formula-to-tvars
+                                (aconst-to-formula eq-compat-rev-aconst)))
+                          (py "boole")))
+            (tpsubst-eq-compat-rev-proof
+             (proof-subst tsubst-eq-compat-rev-proof
+                          (car (formula-to-pvars
+                                (proof-to-formula tsubst-eq-compat-rev-proof)))
+                          cterm))
+            (aux-avar (formula-to-new-avar (make-atomic-formula test-term)))
+            (aux-proof ;M' : all ys.mu ys < mu xs -> GRecGuard
+					;mu ys G (mu ys < mu xs) mr A(ys)
+             (apply
+              mk-proof-in-intro-form
+              (append
+               new-vars
+               (list
+                aux-avar
+                (mk-proof-in-elim-form
+                 tpsubst-eq-compat-rev-proof test-term (pt "True")
+                 (mk-proof-in-elim-form
+                  (make-proof-in-aconst-form
+                   (finalg-to-=-to-eq-aconst (py "boole")))
+                  test-term (pt "True")
+                  (mk-proof-in-elim-form
+                   (make-proof-in-aconst-form atom-true-aconst)
+                   test-term
+                   (make-proof-in-avar-form aux-avar)))
+                 (apply mk-proof-in-elim-form
+                        (cons (make-proof-in-avar-form mr-prog-prem-avar)
+                              (append (map make-term-in-var-form new-vars)
+                                      (list (make-proof-in-avar-form
+                                             aux-avar))))))))))
+            (mr-prog-proof ;M : Prog_mu{xs | GRec mu xs G mr A(xs)}
+             (apply
+              mk-proof-in-intro-form
+              (append
+               vars
+               (list
+                mr-prog-prem-avar
+                (apply
+                 mk-proof-in-elim-form
+                 (cons
+                  (make-proof-in-avar-form prog-avar)
+                  (append
+                   (map make-term-in-var-form vars)
+                   (list
+                    (apply mk-term-in-abst-form
+                           (append new-vars
+                                   (list prev-applied-grecguard-term)))
+                    aux-proof)))))))))
+       (apply
+        mk-proof-in-nc-intro-form
+        (append
+         free
+         (list
+          (apply
+           mk-proof-in-intro-form
+           (cons
+            measure-var
+            (append
+             vars
+             (list
+              real-var
+              prog-avar
+              (apply
+               mk-proof-in-elim-form
+               (append
+                (cons
+                 (make-proof-in-aconst-form mr-gind-aconst)
+                 (map make-term-in-var-form mr-free))
+                (list (make-term-in-var-form measure-var))
+                (map make-term-in-var-form vars)
+                (list mr-prog-proof))))))))))))))
+
 (define (ex-formula-to-ex-intro-mr-proof ex-formula)
   (let ((pvar-to-mr-pvar (make-pvar-to-mr-pvar)))
     (ex-formula-to-ex-intro-mr-proof-aux pvar-to-mr-pvar ex-formula)))
@@ -1715,6 +1918,8 @@
 		 ((string=? "Cases" name)
 		  (all-formula-to-mr-cases-proof-aux
 		   pvar-to-mr-pvar (car (aconst-to-repro-formulas aconst))))
+                 ((string=? "GInd" name)
+                  (gind-aconst-to-mr-gind-proof-aux pvar-to-mr-pvar aconst))
 		 ((string=? "Ex-Intro" name)
 		  (ex-formula-to-ex-intro-mr-proof-aux
 		   pvar-to-mr-pvar (car (aconst-to-repro-formulas aconst))))
