@@ -1,4 +1,4 @@
-; $Id: formula.scm,v 1.76 2007/09/13 10:00:53 schwicht Exp $
+; $Id: formula.scm,v 1.79 2008/01/28 09:20:11 logik Exp $
 ; 7. Formulas and comprehension terms
 ; ===================================
 
@@ -96,6 +96,57 @@
 					  -1 "Total")))
     (make-predicate-formula total-predconst term)))
 
+(define (make-se term)
+  (let* ((type (term-to-type term))
+         (se-term (make-term-in-const-form
+                   (sfinalg-to-se-const type)))) ;includes check for sfinalg
+    (make-atomic-formula (mk-term-in-app-form se-term term))))
+
+(define (make-stotal term)
+  (let* ((type (term-to-type term))
+	 (tvar (make-tvar -1 DEFAULT-TVAR-NAME))
+         (stotal-predconst (make-predconst (make-arity tvar)
+                                           (make-subst tvar type)
+                                           -1 "STotal")))
+    (make-predicate-formula stotal-predconst term)))
+
+(define (make-stotal-or-se term)
+  (let ((type (term-to-type term)))
+    (if
+     (sfinalg? type)
+     (make-atomic-formula
+      (mk-term-in-app-form
+       (make-term-in-const-form
+	(make-const (se-at type)
+		    "SE" 'fixed-rules
+		    (make-arrow type (make-alg "boole")) empty-subst
+		    1 'prefix-op))
+       term))
+     (make-stotal term))))
+
+(define (make-stotal-or-se-or-e term)
+  (let ((type (term-to-type term)))
+    (cond
+     ((finalg? type)
+      (make-atomic-formula
+       (mk-term-in-app-form
+	(make-term-in-const-form
+	 (make-const (e-at type)
+		     "E" 'fixed-rules
+		     (make-arrow type (make-alg "boole")) empty-subst
+		     1 'prefix-op))
+	term)))
+     ((sfinalg? type)
+      (make-atomic-formula
+       (mk-term-in-app-form
+	(make-term-in-const-form
+	 (make-const (se-at type)
+		     "SE" 'fixed-rules
+		     (make-arrow type (make-alg "boole")) empty-subst
+		     1 'prefix-op))
+	term)))
+     (else (make-stotal term)))))
+
 ; Constructor and accessors for formulas:
 
 (define (make-imp premise conclusion) (list 'imp premise conclusion))
@@ -961,7 +1012,16 @@
   (case (tag formula)
     ((atom) (term-to-free (atom-form-to-kernel formula)))
     ((predicate)
-     (apply union (map term-to-free (predicate-form-to-args formula))))
+     (let ((pred (predicate-form-to-predicate formula)))
+       (cond
+	((or (pvar-form? pred) (predconst-form? pred))
+	 (apply union (map term-to-free (predicate-form-to-args formula))))
+	((idpredconst-form? pred)
+	 (let ((cterms (idpredconst-to-cterms pred)))
+	   (apply union
+		  (append (map term-to-free (predicate-form-to-args formula))
+			  (map cterm-to-free cterms)))))
+	(else (myerror "formula-to-pvars" "predicate expected" pred)))))
     ((imp) (union (formula-to-free (imp-form-to-premise formula))
 		  (formula-to-free (imp-form-to-conclusion formula))))
     ((and) (union (formula-to-free (and-form-to-left formula))
@@ -1008,7 +1068,14 @@
     ((atom) '())
     ((predicate)
      (let ((pred (predicate-form-to-predicate formula)))
-       (if (pvar-form? pred) (list pred) '())))
+       (cond
+	((pvar-form? pred) (list pred))
+	((predconst-form? pred) '())
+	((idpredconst-form? pred)
+	 (let* ((cterms (idpredconst-to-cterms pred))
+		(formulas (map cterm-to-formula cterms)))
+	   (apply union (map formula-to-pvars formulas))))
+	(else (myerror "formula-to-pvars" "predicate expected" pred)))))
     ((imp) (union (formula-to-pvars (imp-form-to-premise formula))
 		  (formula-to-pvars (imp-form-to-conclusion formula))))
     ((and) (union (formula-to-pvars (and-form-to-left formula))
