@@ -1,4 +1,4 @@
-; $Id: grammar.scm,v 1.91 2007/07/25 10:23:41 schwicht Exp $
+; $Id: grammar.scm,v 1.93 2008/01/25 13:30:10 logik Exp $
 ; 7-5. Parsing
 ; ============
 
@@ -63,7 +63,7 @@
     ((appterm) 10)
     ((prefix-op preterm) 11)
     ((postfix-op postterm) 12)
-    ((const var number rec-op if-op atomic-term) 13)
+    ((const var number rec-op grec-op grecguard-op if-op atomic-term) 13)
     (else (myerror
 	   \"token-type-to-precendence: token type expected\" token-type))))
 
@@ -130,6 +130,8 @@
      postfix-op
      prefix-op
      rec-op
+     grec-op
+     grecguard-op
      binding-op
      add-op
      mul-op
@@ -247,6 +249,12 @@
      ("Rec" rec-op . (lambda (l)
 		       (make-term-in-const-form
 			(apply type-info-to-rec-const l))))
+     ("GRec" grec-op . (lambda (l)
+		       (make-term-in-const-form
+			(type-info-to-grec-const l))))
+     ("GRecGuard" grecguard-op . (lambda (l)
+		       (make-term-in-const-form
+			(type-info-to-grecguard-const l))))
      ("=" rel-op . (lambda (term1 term2)
 		     (let ((type1 (term-to-type term1))
 		     	   (type2 (term-to-type term2)))
@@ -275,6 +283,10 @@
                         (make-term-in-app-form
                           (make-term-in-const-form 
                             (finalg-to-e-const (term-to-type term))) term)))
+     ("SE" prefix-op . (lambda (term) 
+                         (make-term-in-app-form
+                           (make-term-in-const-form 
+                             (sfinalg-to-se-const (term-to-type term))) term)))
      ("@" pair-op . make-term-in-pair-form)
      ("left" prefix-op . make-term-in-lcomp-form)
      ("right" prefix-op . make-term-in-rcomp-form)
@@ -640,6 +652,8 @@
       (number) : (make-numeric-term $1)
       (lpar rec-op typelist rpar) : ($2 $3)
       (lpar rec-op number typelist rpar) : ($2 (cons $3 $4)) ;added 02-01-01
+      (lpar grec-op typelist rpar) : ($2 $3)
+      (lpar grecguard-op typelist rpar) : ($2 $3)
       (lpar constscheme typelist rpar) : 
         (make-term-in-const-form
 	 (let* ((tvars (const-to-tvars $2))
