@@ -1,4 +1,4 @@
-; $Id: pconst.scm,v 1.111 2007/09/06 07:47:26 schwicht Exp $
+; $Id: pconst.scm,v 1.115 2008/01/28 09:20:11 logik Exp $
 ; 4. Constants
 ; ============
 
@@ -121,7 +121,8 @@
 (define (constr-name? string) (assoc string CONSTRUCTORS))
 (define (pconst-name? string) (assoc string PROGRAM-CONSTANTS))
 (define (fixed-rules-name? string)
-  (and (member string '("Rec" "Cases" "Ex-Elim" "=" "E"))))
+  (and (member string '("Rec" "Cases" "GRec" "GRecGuard" "Efq"
+			"Ex-Elim" "=" "E" "SE"))))
 
 (define (const=? const1 const2)
   (and (string=? (const-to-name const1) (const-to-name const2))
@@ -156,8 +157,6 @@
 	(myerror "constr-name-to-inst-constructors" "constructor name expected"
 		 name))))
 
-(define (constr-name? string) (assoc string CONSTRUCTORS))
-
 (define (constr-name-and-tsubst-to-constr name tsubst)
   (let ((info (assoc name CONSTRUCTORS)))
     (if info
@@ -445,11 +444,17 @@
 				"AndConst" "ImpConst" "OrConst" "NegConst")))
 	     (begin
 	       (comment
-		"ok, program constant " name ": " (type-to-string uninst-type))
+		"ok, program constant " name ": "
+		(type-to-string uninst-type))
 	       (if (not (eq? 'const token-type))
 		   (comment "of token type " token-type " and"))
 	       (comment "of t-degree " t-deg " and arity " arity
-			" added"))))))))
+			" added")
+	       (if (and (t-deg-one? t-deg)
+			(not (assoc (string-append name "Total") THEOREMS)))
+		   (comment "warning: theorem "
+			    (string-append name "Total")
+			    " stating totality missing")))))))))
   (let ((l (length (arrow-form-to-arg-types uninst-type))))
     (if 
      (null? rest)
@@ -458,10 +463,75 @@
        (if (null? (cdr rest))
 	   (add-program-constant-aux name uninst-type t-deg 'const l)
 	   (let ((token-type (cadr rest)))
-	     (if (null? (cddr rest))
-		 (add-program-constant-aux name uninst-type t-deg token-type l)
-		 (add-program-constant-aux name uninst-type t-deg token-type
-					   (caddr rest)))))))))
+	     (if
+	      (null? (cddr rest))
+	      (add-program-constant-aux name uninst-type t-deg token-type l)
+	      (add-program-constant-aux name uninst-type t-deg token-type
+					(caddr rest)))))))))
+
+(define (change-t-deg-to-one name)
+  (let ((info (assoc name PROGRAM-CONSTANTS)))
+    (if
+     (not info)
+     (myerror "change-t-deg-to-one" "name of program constant expected" name))
+    (let* ((pconst (pconst-name-to-pconst name))
+	   (comprules (pconst-name-to-comprules name))
+	   (rewrules (pconst-name-to-rewrules name))
+	   (code (pconst-name-to-external-code name)) ;may be #f
+	   (pconsts-exept-name
+	    (do ((l PROGRAM-CONSTANTS (cdr l))
+		 (res '() (if (string=? (caar l) name)
+			      res
+			      (cons (car l) res))))
+		((null? l) (reverse res))))
+	   (arity (const-to-object-or-arity pconst))
+	   (uninst-type (const-to-uninst-type pconst))
+	   (token-type (const-to-token-type pconst))
+	   (new-pconst (make-const arity name 'pconst uninst-type empty-subst
+				   t-deg-one token-type))
+	   (obj
+	    (if (zero? arity)
+		(nbe-reflect
+		 (nbe-make-termfam
+		  uninst-type
+		  (lambda (k) (make-term-in-const-form new-pconst))))
+		(nbe-make-object
+		 uninst-type
+		 (nbe-curry
+		  (lambda objs ;arity many
+		    (let* ((obj1 (nbe-reflect
+				  (nbe-make-termfam
+				   uninst-type
+				   (lambda (k)
+				     (make-term-in-const-form new-pconst)))))
+			   (val (nbe-object-to-value obj1)))
+		      (apply (nbe-uncurry val arity) objs)))
+		  uninst-type
+		  arity))))
+	   (inst-objs (list (list empty-subst obj))))
+      (set! PROGRAM-CONSTANTS
+	    (cons (if code
+		      (list name new-pconst '() '() inst-objs code)
+		      (list name new-pconst '() '() inst-objs))
+		  pconsts-exept-name))
+      (remove-token name)
+      (if (null? (type-to-free uninst-type))
+	  (add-token name
+		     (const-to-token-type pconst)
+		     (const-to-token-value new-pconst))
+	  (add-token name
+		     'constscheme
+		     new-pconst))
+      (do ;add again the previous comprules, now using the new-pconst
+	  ((lc comprules (cdr lc)))
+	  ((null? lc))
+	(add-computation-rule (pt (term-to-string (rule-to-lhs (car lc))))
+			      (pt (term-to-string (rule-to-rhs (car lc))))))
+      (do ;add again the previous rewrules, now using the new-pconst
+	  ((lr rewrules (cdr lr)))
+	  ((null? lr))
+	(add-rewrite-rule (pt (term-to-string (rule-to-lhs (car lr))))
+			  (pt (term-to-string (rule-to-rhs (car lr)))))))))
 
 (define apc add-program-constant)
 
@@ -538,31 +608,42 @@
 	     (comprules (pconst-name-to-comprules name))
 	     (rewrules (pconst-name-to-rewrules name))
 	     (arity (const-to-object-or-arity pconst))
-	     (renamed-lhs
+	     (renamed-lhs-and-rhs
 	      (if (zero? arity)
 		  (let* ((lhs-tvars (const-to-tvars pconst))
 			 (new-tvars (map (lambda (x) (new-tvar)) lhs-tvars))
 			 (new-tsubst
 			  (map (lambda (x y) (list x y)) lhs-tvars new-tvars)))
-		    (term-substitute lhs new-tsubst))
+		    (list (term-substitute lhs new-tsubst)
+			  (term-substitute rhs new-tsubst)))
 		  (let* ((newvars (map (lambda (x) (make-term-in-var-form
 						    (var-to-new-var x)))
 				       lhsfree))
 			 (subst (map (lambda (x y) (list x y))
 				     lhsfree newvars)))
-		    (term-substitute lhs subst)))))
+		    (list (term-substitute lhs subst)
+			  (term-substitute rhs subst)))))
+	     (renamed-lhs (car renamed-lhs-and-rhs))
+	     (renamed-rhs (cadr renamed-lhs-and-rhs)))
 	(for-each ;of comprules
 	 (lambda (cr)
-	   (let ((old-lhs (car cr)))
+	   (let ((old-lhs (car cr))
+		 (old-rhs (cadr cr)))
 	     (if
 	      (if (zero? arity)
-		  (type-unify (term-to-type old-lhs)
-			      (term-to-type renamed-lhs))
-		  (unify old-lhs renamed-lhs))
+		  (let ((tunif (type-unify (term-to-type old-lhs)
+					   (term-to-type renamed-lhs))))
+		    (and tunif
+			 (not (term=? (term-substitute old-rhs tunif)
+				      (term-substitute renamed-rhs tunif)))))
+		  (let ((unif (unify old-lhs renamed-lhs)))
+		    (and unif
+			 (not (term=? (term-substitute old-rhs unif)
+				      (term-substitute renamed-rhs unif))))))
 	      (myerror
-	       "add-computation-rule" "lhs" lhs
-	       "overlaps with lhs of already existing computation rule for"
-	       old-lhs))))
+	       "add-computation-rule" lhs "->" rhs
+	       "is in conflict with already existing computation rule"
+	       old-lhs "->" old-rhs))))
 	 comprules)
 	(let* ((new-comprules
 		(append comprules
@@ -1557,21 +1638,6 @@
 		       (- (length type-info-or-repro-formulas) 1))
 	    '()))))
 
-; Code discarded 2007-07-11
-; (define (cases-const-to-param-types const)
-;   (let ((type-info-or-repro-formulas
-; 	 (const-to-type-info-or-repro-formulas const)))
-;     (if (formula-form? (car type-info-or-repro-formulas))
-; 	(let* ((repro-formulas type-info-or-repro-formulas)
-; 	       (free-lists (map formula-to-free repro-formulas))
-; 	       (free (apply union free-lists)))
-; 	  (map var-to-type free))
-; 	(let* ((type-info type-info-or-repro-formulas)
-; 	       (fst (car type-info)))
-; 	  (if (integer? fst)
-; 	      (list-head (cdr type-info) fst)
-; 	      '())))))
-
 (define (arrow-type-to-uninst-paramless-casesop-type-and-tsubst arrow-type)
   (let* ((arg-type (arrow-form-to-arg-type arrow-type))
 	 (val-type (arrow-form-to-val-type arrow-type))
@@ -1825,6 +1891,377 @@
 			 alg-names-with-uninst-recop-types)
 		   imp-formulas))))
 
+; General recursion and induction (work by Simon Huber)
+
+; (GRecGuard rhos tau) :
+; (rhos=>tau)=>rhos=>(rhos=>(rhos=>tau)=>tau)=>boole=>tau
+; GRecGuard mu xs G True -> G xs ([ys] GRecGuard mu ys G (mu ys < mu xs))
+; GRecGuard mu xs G False -> Inhab
+; For convenience we add GRec with GRec mu xs G -> GRecGuard mu xs G True
+
+; (GRecGuard m alphas rhos tau) :
+; alphas=>(rhos=>tau)=>rhos=>(rhos=>(rhos=>atomic=>tau)=>tau)=>
+; boole=>atomic=>tau
+; GRecGuard ts mu xs G True u ->
+; G xs ([ys,atomic] GRecGuard ts mu ys G (mu ys < mu xs) atomic)
+; GRecGuard ts mu xs G False u -> Efq u
+; Note that this variant is only used to normalize proofs.  Here we need
+; that Efq is a constant.
+
+; Induction
+; GInd : allnc zs all mu, xs(Prog_mu{xs|A(xs)} ->
+;          all boole(atom(boole) -> A(xs))), where
+; Prog_mu{xs|A(xs)} = all xs(all ys(mu ys < mu xs -> A(ys)) -> A(xs))
+; We get the ordinary general induction GInd' by:
+; GInd' ts mu xs M = GInd ts mu xs M True Truth-Axiom
+
+(define (grecguard-const-to-param-types const)
+  (let ((repro-formulas (const-to-type-info-or-repro-formulas const)))
+    (if (and (pair? repro-formulas)
+             (formula-form? (car repro-formulas)))
+        (let* ((all-formula (car repro-formulas))
+               (free (formula-to-free all-formula)))
+          (map var-to-type free))
+        '())))
+
+; grecguard-const-for-gind? returns true if the step type of the
+; constant is (rhos=>(rhos=>atomic=>tau)=>tau), i.e., if the constant
+; comes from a GInd aconst.
+
+(define (grecguard-const-for-gind? const)
+  (let* ((type (const-to-type const))
+         (f (length (grecguard-const-to-param-types const)))
+         (type-wo-params (arrow-form-to-final-val-type type f))
+         (measure-type (arrow-form-to-arg-type type-wo-params))
+         (arg-types (arrow-form-to-arg-types measure-type))
+         (m (length arg-types))
+         (step-type (arrow-form-to-arg-type
+                     (arrow-form-to-final-val-type type-wo-params (+ m 1))))
+         (aux-type (arrow-form-to-arg-type
+                    (arrow-form-to-final-val-type step-type m)))
+         (aux-arg-types (arrow-form-to-arg-types aux-type))
+         (len (length aux-arg-types)))
+    (= (+ m 1) len)))
+
+(define (type-etc-to-efq-const f uninst-type inst-type tsubst . repro-formula)
+  (apply
+   make-const
+   (append (list (apply efq-at (append (list f uninst-type inst-type tsubst)
+                                       repro-formula))
+                 "Efq" 'fixed-rules uninst-type tsubst t-deg-one 'const)
+           repro-formula)))
+
+; formula-to-efq-const will be used in proof.scm to achieve
+; normalization of proofs via translating them in terms, to translate
+; an efq-aconst.  In addition we need the number m of quantifiers used
+; for the axioms.
+
+(define (formula-to-efq-const formula)
+  (let* ((efqaconst (global-assumption-name-to-aconst "Efq"))
+         (efq-uninst-formula (aconst-to-uninst-formula efqaconst))
+         (pvar (predicate-form-to-predicate
+                (imp-form-to-conclusion efq-uninst-formula)))
+         (tvar (PVAR-TO-TVAR pvar))
+         (nbe-type (nbe-formula-to-type formula))
+         (tsubst (make-subst tvar nbe-type))
+         (free (formula-to-free formula))
+         (param-types (map var-to-type free))
+         (f (length free))
+         (uninst-type (apply mk-arrow
+                             (append param-types
+                                     (list (make-tconst "atomic") tvar))))
+         (inst-type (type-substitute uninst-type tsubst)))
+    (type-etc-to-efq-const f uninst-type inst-type tsubst formula)))
+
+(define (efq-at f uninst-type inst-type tsubst . repro-formula)
+  (nbe-make-object
+   inst-type
+   (nbe-curry
+    (lambda objs
+      (nbe-reflect ;reproduce
+       (nbe-make-termfam
+        (arrow-form-to-final-val-type inst-type (+ f 1))
+        (lambda (k)
+          (apply mk-term-in-app-form
+                 (cons
+                  (make-term-in-const-form
+                   (type-etc-to-efq-const f uninst-type inst-type
+                                          tsubst (car repro-formula)))
+                  (map (lambda (x) (nbe-fam-apply (nbe-reify x) k)) objs)))))))
+    inst-type (+ f 1))))
+
+(define (type-info-to-grec-type type-info)
+  (if
+   (< (length type-info) 2)
+   (myerror "type-info-to-grec-type"
+	    "more types expected"
+	    type-info)
+   (let* ((m (- (length type-info) 1))
+	  (arg-types (list-head type-info m))
+	  (val-type (list-ref type-info m))
+	  (measure-type
+	   (apply mk-arrow (append arg-types (list (py "nat")))))
+	  (step-type
+	   (apply mk-arrow
+		  (append
+		   arg-types
+		   (list (apply mk-arrow (append arg-types (list val-type)))
+			 val-type)))))
+     (apply mk-arrow
+	    (cons measure-type
+		  (append arg-types (list step-type val-type)))))))
+
+(define (type-info-to-grecguard-type type-info)
+  (if (< (length type-info) 2)
+      (myerror "type-info-to-grecguard-type"
+               "more types expected"
+               type-info)
+      (let* ((gind-type? (number? (car type-info)))
+             (numparams-and-types
+              (if gind-type?
+                  type-info
+                  (cons 0 type-info)))
+             (numparams (car numparams-and-types))
+             (types (cdr numparams-and-types))
+             (param-types (list-head types numparams)) ;alphas
+             (revmtypes (reverse (list-tail types numparams)))
+             (arg-types (reverse (cdr revmtypes))) ;rhos
+             (val-type (car revmtypes)) ;tau
+             (measure-type
+              (apply mk-arrow (append arg-types (list (py "nat")))))
+             (step-type
+              (apply mk-arrow
+                     (append arg-types
+                             (list
+                              (apply
+                               mk-arrow
+                               (append
+                                arg-types
+                                (if gind-type?
+                                    (list (make-tconst "atomic"))
+                                    '())
+                                (list val-type)))
+                              val-type)))))
+        (apply mk-arrow
+               (append param-types
+                       (cons measure-type
+                             (append
+                              arg-types
+                              (list step-type (py "boole"))
+                              (if gind-type?
+                                  (list (make-tconst "atomic"))
+                                  '())
+                              (list val-type))))))))
+
+; In type-info-etc-to-grec-const type-info is (rhos tau).  The case (f
+; alphas rhos tau) (f = |alphas|) is not allowed here.
+
+(define (type-info-etc-to-grec-const type-info tsubst)
+  (let* ((m (- (length type-info) 1))
+         (uninst-grecop-type (type-info-to-grec-type type-info))
+         (inst-grecop-type (type-substitute uninst-grecop-type tsubst)))
+    (make-const
+     (grec-at  m inst-grecop-type type-info tsubst)
+     "GRec" 'fixed-rules uninst-grecop-type tsubst t-deg-one 'const)))
+
+(define (type-info-to-grec-const type-info)
+  (type-info-etc-to-grec-const type-info empty-subst))
+
+; In type-info-etc-to-grecguard-const type-info is either (rhos tau)
+; or (f alphas rhos tau) (f = |alphas|).
+
+(define (type-info-etc-to-grecguard-const type-info tsubst . all-formula)
+  (let* ((f (if (number? (car type-info))
+                (car type-info)
+                0))
+         (m (if (number? (car type-info))
+                (- (length type-info) f 2)
+                (- (length type-info) 1)))
+         (uninst-grecguardop-type (type-info-to-grecguard-type type-info))
+         (inst-grecguardop-type
+          (type-substitute uninst-grecguardop-type tsubst)))
+    (apply
+     make-const
+     (append
+      (list (apply grecguard-at
+                   (append (list f m inst-grecguardop-type type-info tsubst)
+                           all-formula))
+            "GRecGuard" 'fixed-rules
+            uninst-grecguardop-type tsubst t-deg-one 'const)
+      all-formula))))
+
+(define (type-info-to-grecguard-const type-info)
+  (type-info-etc-to-grecguard-const type-info empty-subst))
+
+; While normalizing, the unfolding mechanism of the grecguard-operator
+; may be blocked by setting GRECGUARD-UNFOLDING-FLAG to #f.
+
+(define GRECGUARD-UNFOLDING-FLAG #t)
+(define INITIAL-GRECGUARD-UNFOLDING-FLAG GRECGUARD-UNFOLDING-FLAG)
+
+; grecguard-at computes the object for nbe.  Here f is the number of
+; "free" parameter objs and m is the number of arguments for the
+; measure function.
+
+(define (grecguard-at f m inst-grecguardop-type type-info tsubst . all-formula)
+  (let* ((gind-type? (if (number? (car type-info))
+                         #t
+                         #f))
+         (arity (if gind-type? (+ f m 4) (+ m 3))))
+    (nbe-make-object
+     inst-grecguardop-type
+     (nbe-curry
+      (lambda objs
+        (let* ((test-obj (list-ref objs (+ f m 2))) ;b
+               (test-val (nbe-object-to-value test-obj))
+               (param-objs (list-head objs f))
+               (measure-obj (list-ref objs f)) ;mu
+               (step-obj (list-ref objs (+ f m 1))) ;G
+               (arg-objs (list-head (list-tail objs (+ f 1)) m)) ;xs
+               (arg-types (map nbe-object-to-type arg-objs))
+               (val-type ;tau
+                (arrow-form-to-final-val-type inst-grecguardop-type arity)))
+          (cond
+           ((or (nbe-fam-value? test-val)
+		(not GRECGUARD-UNFOLDING-FLAG)) ;reproduce
+            (let ((grecguard-term
+                   (make-term-in-const-form
+                    (apply type-info-etc-to-grecguard-const
+			   (append (list type-info tsubst) all-formula)))))
+              (nbe-reflect
+               (nbe-make-termfam
+                val-type
+                (lambda (k)
+                  (apply
+                   mk-term-in-app-form
+                   (cons grecguard-term
+                         (map (lambda (x)
+                                (nbe-fam-apply
+                                 (nbe-reify x) k))
+                              objs))))))))
+           ((nbe-constr-value? test-val)
+            (let ((name (nbe-constr-value-to-name test-val)))
+              (cond
+               ((string=? name "True")
+                (let* ((pd-type (apply mk-arrow
+                                       (append
+                                        arg-types
+                                        (if gind-type?
+                                            (list (make-tconst "atomic"))
+                                            '())
+                                        (list val-type))))
+                       (pd-obj ;([ys] GRecGuard mu ys G (mu ys < mu xs))
+					;or ([ys,atomic] ...)
+                        (nbe-make-object
+                         pd-type
+                         (nbe-curry
+                          (lambda objs2
+                            (let* ((ys (list-head objs2 m))
+                                   (muys (apply nbe-object-app
+                                                (cons measure-obj ys)))
+                                   (muxs (apply nbe-object-app
+                                                (cons measure-obj arg-objs)))
+                                   (test-obj ;mu ys < mu xs
+                                    (nbe-object-app natlt-obj muys muxs))
+                                   (grecguard-const
+                                    (apply
+                                     type-info-etc-to-grecguard-const
+                                     (append (list type-info tsubst)
+                                             all-formula)))
+                                   (grecguard-obj (const-to-object-or-arity
+						   grecguard-const)))
+                              (apply
+                               nbe-object-app
+                               (cons grecguard-obj
+                                     (append param-objs (cons measure-obj ys)
+                                             (list step-obj test-obj)
+                                             (if gind-type?
+                                                 (list (list-ref objs2 m))
+                                                 '()))))))
+                          pd-type
+                          (if gind-type? (+ m 1) m)))))
+                  (apply nbe-object-app
+                         (cons step-obj (append arg-objs (list pd-obj))))))
+               ((string=? name "False")
+                (if gind-type?
+                    (let* ((vars (all-form-to-vars (car all-formula) m))
+                           (kernel
+                            (all-form-to-final-kernel (car all-formula) m))
+                           (free (formula-to-free (car all-formula)))
+                           (arg-terms (map (lambda (obj)
+                                             (nbe-extract (nbe-reify obj)))
+                                           arg-objs))
+                           (subst (make-substitution vars arg-terms))
+                           (inst-formula (formula-substitute kernel subst))
+                           (subst-all-formula
+                            (apply mk-all (append free (list inst-formula))))
+                           (new-free (formula-to-free subst-all-formula))
+                           (new-free-objs
+                            (map (lambda (x)
+                                   (nbe-reflect
+                                    (nbe-term-to-termfam
+                                     (make-term-in-var-form x))))
+                                 new-free))
+                           (new-all-formula
+                            (apply mk-all (append
+                                           new-free
+                                           (list subst-all-formula)))) ;closed
+                           (efqconst (formula-to-efq-const new-all-formula))
+                           (efqobj (const-to-object-or-arity efqconst))
+                           (falsum-obj (list-ref objs (+ f m 3))))
+                      (apply
+                       nbe-object-app
+                       (append (list efqobj falsum-obj)
+                               new-free-objs param-objs)))
+                    (let ((canon-term ;eps
+                           (type-to-canonical-inhabitant val-type)))
+                      (nbe-term-to-object canon-term '()))))
+               (else (myerror "grecguard-at" "True or False expected"
+			      test-name)))))
+           (else (myerror "grecguard-at" "value expected" test-val)))))
+      inst-grecguardop-type
+      arity))))
+
+; We define GRec in terms of GRecGuard
+
+(define (grec-at m inst-grecop-type type-info tsubst)
+  (nbe-make-object
+   inst-grecop-type
+   (nbe-curry
+    (lambda objs
+      (let* ((trueobj (nbe-term-to-object (pt "True") '()))
+             (grecguard-const
+              (type-info-etc-to-grecguard-const type-info tsubst))
+             (grecguard-obj (const-to-object-or-arity grecguard-const)))
+        (apply
+         nbe-object-app (cons grecguard-obj (append objs (list trueobj))))))
+    inst-grecop-type
+    (+ m 2))))
+
+; all-formula-to-grecguard-const will be used in proof.scm to achieve
+; normalization of proofs via translating them in terms, to translate
+; a gind-aconst.  In addition we need the number m of quantifiers used
+; for the axioms.
+
+(define (all-formula-to-grecguard-const all-formula m)
+  (let* ((uninst-gind-formula-and-tpinst
+          (all-formula-to-uninst-gind-formula-and-tpinst all-formula m))
+	 (tpinst (cadr uninst-gind-formula-and-tpinst))
+	 (pinst (list-transform-positive tpinst
+		  (lambda (x) (pvar-form? (car x)))))
+	 (pvar (caar pinst)) ;there is only one pvar (gind is not simult.)
+	 (cterm (cadar pinst))
+	 (new-tvar (PVAR-TO-TVAR pvar))
+	 (nbe-type (nbe-formula-to-type (cterm-to-formula cterm)))
+	 (new-tsubst (make-subst new-tvar nbe-type))
+	 (free (formula-to-free all-formula))
+	 (param-types (map var-to-type free)) ;alphas
+	 (f (length free))
+	 (vars (all-form-to-vars all-formula m))
+	 (types (map var-to-type vars)) ;rhos
+	 (type-info (cons f (append param-types types (list nbe-type)))))
+    (type-info-etc-to-grecguard-const type-info new-tsubst all-formula)))
+
 ; ex-elim: allnc zs.ex z A -> (all z.A -> B) -> B
 
 (define (ex-formula-and-concl-to-ex-elim-const ex-formula concl)
@@ -2110,6 +2547,13 @@
 		    "Cases" 'fixed-rules uninst-type restricted-tsubst
 		    1 'const)
 	      type-info-or-repro-formulas))))
+	 ((string=? "SE" name)
+	  (let* ((inst-type (type-substitute uninst-type restricted-tsubst))
+		 (sfinalg (arrow-form-to-arg-type inst-type)))
+	    (make-const (se-at sfinalg)
+			"SE" 'fixed-rules
+			(make-arrow sfinalg (make-alg "boole")) empty-subst
+			1 'prefix-op)))
 	 ((string=? "=" name) const)
 	 ((string=? "E" name) const)
 	 ((string=? "Ex-Elim" name) const)
