@@ -1,4 +1,4 @@
-; $Id: psym.scm,v 1.45 2007/08/24 08:16:35 schwicht Exp $
+; $Id: psym.scm,v 1.48 2008/01/28 09:20:10 logik Exp $
 ; 5. Predicates
 ; =============
 
@@ -769,13 +769,9 @@
 			    (list ")"))))))))))
 
 (define (idpredconst-to-free idpc)
-  (let* ((name (idpredconst-to-name idpc))
-	 (types (idpredconst-to-types idpc))
-	 (param-cterms (idpredconst-to-cterms idpc))
-	 (tsubst (idpredconst-name-and-types-to-tsubst name types))
-	 (subst-param-cterms (map (lambda (x) (cterm-substitute x tsubst))
-				  param-cterms)))
-    (apply union (map cterm-to-free subst-param-cterms))))
+  (let* ((types (idpredconst-to-types idpc))
+	 (cterms (idpredconst-to-cterms idpc)))
+    (apply union (map cterm-to-free cterms))))
 
 ; Inductively defined predicate constant names are provided in the form
 ; of an association list IDS, which assigns all relevant information to
@@ -915,7 +911,10 @@
 					     (not (is-used? s '()
 							    'idpredconst))))
 				      idpc-names)))
-	      (myerror "add-ids" "list of new strings expected" idpc-names)))
+	      (begin
+		(set! COMMENT-FLAG OLD-COMMENT-FLAG)
+		(myerror "add-ids" "list of new strings expected"
+			 idpc-names))))
 	 (clause-strings-with-opt-names-test
 	  (for-each
 	   (lambda (x)
@@ -923,9 +922,11 @@
 		     (< 2 (length x))
 		     (not (string? (car x)))
 		     (and (pair? (cdr x)) (not (string? (cadr x)))))
-		 (myerror "add-ids"
-			  "list of clause-string and optional name expected"
-			  x)))
+		 (begin
+		   (set! COMMENT-FLAG OLD-COMMENT-FLAG)
+		   (myerror "add-ids"
+			    "list of clause-string and optional name expected"
+			    x))))
 	   clause-strings-with-opt-names))
 	 (arities (map cadr idpc-names-with-arities-and-opt-alg-names))
 	 (arities-cc (map cadr idpc-names-with-arities-and-opt-alg-names-cc))
@@ -991,7 +992,9 @@
 		    (concl (imp-form-to-final-conclusion kernel)))
 	       (if (not (and (predicate-form? concl)
 			     (pvar-form? (predicate-form-to-predicate concl))))
-		   (myerror "add-ids" "illegal clause" x))))
+		   (begin
+		     (set! COMMENT-FLAG OLD-COMMENT-FLAG)
+		     (myerror "add-ids" "illegal clause" x)))))
 	   clauses-with-new-pvars))
 	 (clauses-with-new-pvars-and-opt-names
 	  (map (lambda (x y) (cons x y))
@@ -1323,15 +1326,22 @@
 
 (define (remove-idpc-name . x)
   (define (rin1 idpc-name)
-    (set! IDS (list-transform-positive IDS
-		(lambda (x) (not (string=? (car x) idpc-name)))))
-    (set! OLD-COMMENT-FLAG COMMENT-FLAG)
-    (set! COMMENT-FLAG #f)
-    (remove-alg-name (idpredconst-name-to-nbe-alg-name idpc-name))
-    (remove-token idpc-name)
-    (set! COMMENT-FLAG OLD-COMMENT-FLAG)
-    (comment
-     "ok, inductively defined predicate constant " idpc-name " removed"))
+    (let* ((simidpc-names
+	    (idpredconst-name-to-simidpc-names idpc-name))
+	   (opt-alg-names
+	    (apply union
+		   (map (lambda (n) (idpredconst-name-to-opt-alg-name n))
+			simidpc-names))))
+      (set! OLD-COMMENT-FLAG COMMENT-FLAG)
+      (set! COMMENT-FLAG #f)
+      (if (pair? opt-alg-names) (remove-alg-name (car opt-alg-names)))
+      (remove-alg-name (idpredconst-name-to-nbe-alg-name idpc-name))
+      (for-each remove-token simidpc-names)
+      (set! COMMENT-FLAG OLD-COMMENT-FLAG)
+      (set! IDS (list-transform-positive IDS
+		  (lambda (x) (not (member (car x) simidpc-names)))))
+      (comment
+       "ok, inductively defined predicate constant " idpc-name " removed")))
   (for-each (lambda (idpc-name)
 	      (if (not (assoc idpc-name IDS))
 		  (myerror "remove-idpc-name" "idpc name expected" idpc-name)))
@@ -1342,37 +1352,6 @@
 ; the following character is neither ^ nor alphabetic and the
 ; preceding character is not alphabetic.
 
-; Code discarded 2007-07-24
-; (define (append-hat string name)
-;   (let* ((lh (string-length name))
-; 	 (string-list (string->list string))
-; 	 (name-list (string->list name)))
-;     (if (not (apply and-op (map char-alphabetic? name-list)))
-; 	(myerror "append-hat-beta" "alphabetic name expected" name))
-;     (do ((l-and-res-and-prev
-;           (list string-list '() #\space)
-;           (let* ((l (car l-and-res-and-prev))
-;                  (res (cadr l-and-res-and-prev))
-;                  (prev (caddr l-and-res-and-prev))
-;                  (ss (list->string (list-head l lh)))
-;                  (next (if (< lh (length l))
-;                            (list-ref l lh)
-;                            #\space)))
-;             (if (and (string=? ss name)
-;                      (not (char-alphabetic? next))
-;                      (not (char-alphabetic? prev))
-; 		     (not (char=? #\^ next)))
-;                 (list (list-tail l lh)
-;                       (append res name-list (list #\^))
-;                       (car l))
-;                 (list (cdr l) (append res (list (car l))) (car l))))))
-;         ((or (null? (car l-and-res-and-prev))
-;              (< (length (car l-and-res-and-prev)) lh))
-;          (if (null? (car l-and-res-and-prev))
-;              (list->string (cadr l-and-res-and-prev))
-;              (list->string (append (cadr l-and-res-and-prev)
-;                                    (car l-and-res-and-prev))))))))
-
 (define (number-to-alphabetic-string i)
   (do ((charlist (reverse (string->list (number-to-string i))) (cdr charlist))
        (res '() (append (let ((char (car charlist)))
