@@ -1,4 +1,4 @@
-; $Id: typ.scm,v 1.83 2007/09/07 06:29:23 schwicht Exp $
+; $Id: typ.scm,v 1.86 2008/01/28 09:20:11 logik Exp $
 ; 2. Types
 ; ========
 
@@ -936,9 +936,9 @@
 ; defined) constant.
 
 ; The standard example for a finitary free algebra is the type nat of
-; unary natural numbers, with constructors Uzero and Usucc.  Objects of
+; unary natural numbers, with constructors Zero and Succ.  Objects of
 ; type nat are essentially 
-; - Uzero or lists (Usucc obj) 
+; - Zero or lists (Succ obj) 
 ; - term families
 
 (define ALGEBRAS '())
@@ -986,26 +986,51 @@
 	  (names (if (assoc name ALGEBRAS)
 		     (alg-name-to-simalg-names name)
 		     (myerror "finalg?" "alg name expected" name)))
-	  (typed-constr-names (alg-name-to-typed-constr-names name))
+	  (typed-constr-names
+           (apply union (map alg-name-to-typed-constr-names names)))
 	  (constr-types (map typed-constr-name-to-type typed-constr-names))
 	  (argtypes (apply union (map arrow-form-to-arg-types constr-types)))
 	  (tsubst (map (lambda (x y) (list x y))
 		       (alg-name-to-tvars name)
 		       (alg-form-to-types type)))
 	  (tsubst-argtypes (map (lambda (x) (type-substitute x tsubst))
-				argtypes)))
-     (letrec ((test (lambda (type1)
-		      (case (tag type1)
-			((tvar tconst) #f)
-			((alg)
-			 (or (member (alg-form-to-name type1) names)
-			     (finalg? type1)))
-			((arrow) 
-			 (and (equal? (make-alg "unit")
-				      (arrow-form-to-arg-type type1))
-			      (test (arrow-form-to-val-type type1))))
-			(else (myerror "finalg?" "type expected" type1))))))
-       (apply and-op (map test tsubst-argtypes))))))
+				argtypes))
+          (test (lambda (type1)
+                  (if (alg-form? type1)
+                      (or (member (alg-form-to-name type1) names)
+                          (finalg? type1))
+                      #f))))
+     (apply and-op (map test tsubst-argtypes)))))
+
+; sfinalg? checks whether type is a structure-finitary algebra.
+
+(define (sfinalg? type)
+  (and
+   (alg-form? type)
+   (let* ((name (alg-form-to-name type))
+          (names (if (assoc name ALGEBRAS)
+		     (alg-name-to-simalg-names name)
+		     (myerror "sfinalg?" "alg name expected" name)))
+          (typed-constr-names ;all constructors are relevant
+           (apply union (map alg-name-to-typed-constr-names names)))
+          (constr-types (map typed-constr-name-to-type typed-constr-names))
+          (argtypes-lists (map arrow-form-to-arg-types constr-types))
+          (not-param-type? ;filters non parameter types
+           (lambda (x)
+             (let ((val-type (arrow-form-to-final-val-type x)))
+               (if (alg-form? val-type)
+                   (member (alg-form-to-name val-type) names)
+                   #f))))
+          (argtypes-lists-wo-params
+           (map (lambda (lst)
+                  (list-transform-positive lst not-param-type?))
+                argtypes-lists))
+          (argargtypes-lists
+           (map (lambda (lst)
+                  (apply union (map arrow-form-to-arg-types lst)))
+                argtypes-lists-wo-params))
+          (argargtypes-union (apply union argargtypes-lists)))
+     (null? argargtypes-union))))
 
 ; To introduce simultaneous free algebras we use add-param-algs.  Example
 ; (add-param-algs (list "labtree" "labtlist") 'alg-typeop 2
